
#include "AudioController.h"
#include "AudioTrackProcessor.h"
#include "AudioFileProcessor.h"


AudioController::AudioController(double sampling_rate)
    : main_processor_(new juce::AudioProcessorGraph()), audio_play_head_(sampling_rate) {
}

AudioController::~AudioController() {

}

void AudioController::PrepareToPlay(int samples_per_block_expected, double sample_rate) {
    if (main_processor_ == nullptr) {
        return;
    }

    main_processor_->setPlayConfigDetails(0, 2, sample_rate, samples_per_block_expected);
    audio_play_head_.ChangeSamplingRate(sample_rate);

    // initialize graph
    main_processor_->setPlayHead(&audio_play_head_);
    audio_output_node_ =
        main_processor_->addNode(std::make_unique<AudioGraphIOProcessor>(AudioGraphIOProcessor::audioOutputNode));

    auto processor = std::unique_ptr<AudioProcessor>(new AudioTrackProcessor());
    plugin_node_ = main_processor_->addNode(std::move(processor));

	if (plugin_node_ != nullptr && audio_output_node_ != nullptr) {
		for (int channel(0); channel < 2; ++channel) {
			main_processor_->addConnection(
				{ {plugin_node_->nodeID, channel}, {audio_output_node_->nodeID, channel} });
		}
	}

    main_processor_->prepareToPlay(sample_rate, samples_per_block_expected);
}

void AudioController::ProcessBlock(const AudioSourceChannelInfo& buffer_to_fill) {
    if (main_processor_ == nullptr) {
        return;
    }

    if (!main_processor_->isSuspended()) {
        auto* output_buffer(buffer_to_fill.buffer);
        if (output_buffer == nullptr) {
            return;
        }

        MidiBuffer dummy;  // not used
        main_processor_->processBlock(*output_buffer, dummy);
    }

    if (!audio_play_head_.IsPlaying()){
        return;
    }

    audio_play_head_.MoveTimeInSamples(buffer_to_fill.numSamples);
}

void AudioController::TransportPlay(bool should_start_playing, NotificationType notification) {
    audio_play_head_.transportPlay(should_start_playing);
}

void AudioController::MovePosition(MoveType type) {
    switch (type) {
        case IPlayControl::TO_START:
            audio_play_head_.SetTimeInSamples(0);
            break;
        default:
            break;
    }
}

void AudioController::SetPlayPosition(double new_time_in_seconds) {
    audio_play_head_.SetTimeInSamples(static_cast<int64>(new_time_in_seconds * main_processor_->getSampleRate()));
}
