
#include <JuceHeader.h>
#include "ControlPanel.h"
#include "IPlayControl.h"

const String play_svg = R"(
<svg width="100" height="100">
    <path d="M0 0 L100 50 L0 100 Z" style="fill:white"></path>
</svg>
)";

const String stop_svg = R"(
<svg width="100" height="100">
    <path d="M10 10 L90 10 L90 90 L10 90 Z" style="fill:white"></path>
</svg>
)";

const String to_start_svg = R"(
<svg width="100" height="100">
    <path d="M100 0 L10 50 L100 100 Z" style="fill:white"></path>
    <path d="M0 0 L10 0 L10 100 L0 100 Z" style="fill:white"></path>
</svg>
)";

//==============================================================================
ControlPanel::ControlPanel(IPlayControl& play_controller)
	: play_controller_(play_controller) {
	auto InitializeButton = [this](std::unique_ptr<DrawableButton>& button, const String& name, const String& svg) {
		button.reset(new DrawableButton(name, DrawableButton::ImageOnButtonBackground));
		if (auto svg_xml = juce::parseXML(svg)) button->setImages(Drawable::createFromSVG(*svg_xml).get());
		addAndMakeVisible(button.get());

		constexpr int connected_edge_flags(Button::ConnectedOnLeft | Button::ConnectedOnRight | Button::ConnectedOnTop |
			Button::ConnectedOnBottom);
		button->setConnectedEdges(connected_edge_flags);
		button->addListener(this);
	};

	InitializeButton(play_button_, "play button", play_svg);
	InitializeButton(stop_button_, "stop button", stop_svg);
	InitializeButton(to_start_button_, "to start button", to_start_svg);

	play_button_->setClickingTogglesState(false);
	stop_button_->setClickingTogglesState(false);
}

ControlPanel::~ControlPanel() {
	play_button_->removeListener(this);
	stop_button_->removeListener(this);
	to_start_button_->removeListener(this);
}

void ControlPanel::paint(juce::Graphics& g) { g.fillAll(Colours::black); }

void ControlPanel::resized() {
	const int length(getHeight());
	constexpr int num_button(4);
	constexpr int time_w(100);

	int x = (getWidth() - num_button * length - time_w) / 2;

	if (to_start_button_ != nullptr) to_start_button_->setBounds(x, 0, length, length);

	x += length;
	if (stop_button_ != nullptr) stop_button_->setBounds(x, 0, length, length);

	x += length;
	if (play_button_ != nullptr) play_button_->setBounds(x, 0, length, length);
}

void ControlPanel::buttonClicked(Button* button) {
	if (button == play_button_.get()) {
		play_controller_.TransportPlay(true);
	}
	else if (button == stop_button_.get()) {
		play_controller_.TransportPlay(false);
	}
	else if (button == to_start_button_.get()) {
		play_controller_.MovePosition(IPlayControl::TO_START);
	}
}

