
#include "HostAudioPlayHead.h"

HostAudioPlayHead::HostAudioPlayHead(double sampling_rate)
    : time_in_samples_(0),
      sampling_rate_(sampling_rate),
      is_playing_(false) {}

bool HostAudioPlayHead::getCurrentPosition(CurrentPositionInfo& result) {
    const ScopedLock sl(cs_);
    result.resetToDefault();

    result.timeInSamples = time_in_samples_;
    result.isPlaying     = is_playing_;
    result.timeInSeconds = time_in_samples_ / sampling_rate_;
    return true;
}

void HostAudioPlayHead::transportPlay(bool shouldStartPlaying) {
    const ScopedLock sl(cs_);
    is_playing_ = shouldStartPlaying;
}

juce::int64 HostAudioPlayHead::GetTimeInSamples() const {
    const ScopedLock sl(cs_);
    return time_in_samples_;
}

double HostAudioPlayHead::GetTimeInSeconds() const {
    const ScopedLock sl(cs_);
    return time_in_samples_ / sampling_rate_;
}

void HostAudioPlayHead::SetTimeInSamples(juce::int64 new_value) {
    const ScopedLock sl(cs_);
    time_in_samples_ = new_value;
}

void HostAudioPlayHead::MoveTimeInSamples(juce::int64 delta) {
    const ScopedLock sl(cs_);
    time_in_samples_ += delta;
}

bool HostAudioPlayHead::IsPlaying() const {
    const ScopedLock sl(cs_);
    return is_playing_;
}

void HostAudioPlayHead::ChangeSamplingRate(double new_sampling_rate)
{
    const ScopedLock sl(cs_);
    SetTimeInSamples(GetTimeInSeconds() * new_sampling_rate);
    sampling_rate_ = new_sampling_rate;
}
